package com.persagy.apm.energyalarmstarter.alarmdata.mq;


import com.persagy.apm.energyalarmstarter.alarmdata.mq.model.OrderStateMessage;
import com.persagy.apm.energyalarmstarter.alarmdata.service.OrderStateChangeService;
import com.persagy.apm.energyalarmstarter.alarmdata.utils.StringUtil;
import com.rabbitmq.client.Channel;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.*;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import java.io.IOException;

/**
 * 处理工单消息
 *
 * @author lixing
 * @version V1.0 2021/11/15 12:13 下午
 */
@Slf4j
@Configuration
public class OrderMsgHandler {
    @Autowired
    OrderStateChangeService orderStateChangeService;

    /**
     * 工单状态变化exchange, 下一期升级为对接集成框架
     */
    private final String orderStateExchange = "workorder_state_publish_exchange";
    /**
     * 工单状态变化queue
     */
    private final String orderStateQueue = "order_state_queue";

    @Bean
    public FanoutExchange orderStateExchange() {
        return new FanoutExchange(orderStateExchange);
    }

    @Bean
    public Queue orderStateQueue() {
        return new Queue(orderStateQueue, true);
    }

    @Bean
    public Binding orderStateBinding() {
        return BindingBuilder.bind(orderStateQueue()).to(orderStateExchange());
    }

    @RabbitListener(queues = "order_state_queue")    //监听器监听指定的Queue
    public void processOrderState(String message, Channel channel, Message msg) {
        log.info("============================== Receive:" + message);
        try {
            OrderStateMessage orderStateMessage = StringUtil.transferItemToDTO(message, OrderStateMessage.class);
            log.info("currentThread:{}", Thread.currentThread().getId());
            log.info("order_id: {}", orderStateMessage.getOrder_id());
            log.info("order_state: {}", orderStateMessage.getOrder_state());
            // 根据工单状态消息更新报警记录状态
            orderStateChangeService.updateAlarmWhenOrderStateChange(orderStateMessage);
            // 手动确认消息已消费
            channel.basicAck(msg.getMessageProperties().getDeliveryTag(),false);
        } catch (Exception e) {
            log.error("工单状态消息消费失败: {}", e.getMessage());
            try {
                // 将消费失败的消息移除消息队列，避免重复消费
                channel.basicReject(msg.getMessageProperties().getDeliveryTag(),false);
            } catch (IOException ex) {
                log.error("工单状态消息从队列中移除失败，{}", ex.getMessage());
            }
        }
    }
}
